/*
	config.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "log.h"
#include "cfg.h"
#include "trans.h"


CFG * ph_init_configuration(unsigned char launch_setup,lua_State *ls)
{
CFG *aux;
FILE *f;
FILE *f1;
char res;
char *buffer;
char *buf2;

	ph_log(_("- Initiating configuration: "));

	aux=(struct CFG *)malloc(sizeof(struct CFG));
	if (aux==NULL)
	{
		ph_log_memory_error("config.c","ph_init_configuration","aux",sizeof(struct CFG));
	}
	memset(aux,0,sizeof(struct CFG));

	buffer=(char *)malloc(sizeof(char)*512);
	if (buffer==NULL)
	{
		ph_log_memory_error("config.c","ph_init_configuration","buffer",sizeof(char)*512);
	}
	memset(buffer,0,sizeof(char)*512);
	aux->datadir=buffer;
	
	buf2=(char *)malloc(sizeof(char)*512);
	if (buf2==NULL)
	{
		ph_log_memory_error("config.c","ph_init_configuration","buf2",sizeof(char)*512);
	}
	memset(buf2,0,sizeof(char)*512);
	aux->prefix=buf2;
	
	#ifdef WIN32
		f1=fopen(DFLT_CONFIG_FILE_W32,"rb");
	#else
		f1=fopen(DFLT_CONFIG_FILE_UNIX,"rb");
	#endif
	if (f1!=NULL && !launch_setup)
	{
		aux=ph_get_config_from_cfg(aux,ls);
		fclose(f1);
	}
	else
	{
		/* Comprobar que existe el script de configuracion, si no, poner los valor por defecto */
		f=fopen(DFLT_CFGSCRIPT_PATH,"rb");
		if (f==NULL)
		{
			ph_log(_("Warning: the file %s doesn't exist. Assuming defaults.\n"),DFLT_CFGSCRIPT_PATH);
			aux=ph_set_config_defaults(aux);
			ph_save_config_file(aux);
		}
		else
		{
			/* Si el script de configuracion devuelve algun error poner los valores por defecto, si no
			pillar los datos del script. */
			res=lua_dofile(ls,DFLT_CFGSCRIPT_PATH);
			if (res!=0)
			{
				ph_log(_("Warning: there's a problem with the configuration script. Assuming defaults.\n"));
				aux=ph_set_config_defaults(aux);
				ph_save_config_file(aux);
			}
			else
			{
				aux=ph_get_config_from_script(aux,ls);
				ph_save_config_file(aux);
				ph_log(_("Ok.\n"));
			}
		}
	}
	return aux;
}


void ph_destroy_configuration(CFG *cfg)
{
	free(cfg->prefix);
	free(cfg->datadir);
	free(cfg);
}


CFG * ph_set_config_defaults(CFG *cfg)
{
	strcpy(cfg->datadir,DFLT_DATADIR);
	strcpy(cfg->prefix,DFLT_PREFIX);
	cfg->resolution=DFLT_RESOLUTION;
	cfg->xres=DFLT_XRES;
	cfg->yres=DFLT_YRES;
	cfg->fullscreen=DFLT_FULLSCREEN;
	cfg->stretch=DFLT_STRETCH;
	cfg->stretchx=DFLT_STRETCHX;
	cfg->stretchy=DFLT_STRETCHY;
	cfg->enable_sound=DFLT_ENABLE_SOUND;
	cfg->sfx=DFLT_SFX;
	cfg->music=DFLT_MUSIC;
	cfg->sfxvolume=DFLT_SFXVOLUME;
	cfg->musicvolume=DFLT_MUSICVOLUME;
	cfg->controltype=DFLT_CONTROLTYPE;
	cfg->key_up=DFLT_KEY_UP;
	cfg->key_down=DFLT_KEY_DOWN;
	cfg->key_left=DFLT_KEY_LEFT;
	cfg->key_right=DFLT_KEY_RIGHT;
	cfg->game_type=DFLT_GAME_TYPE;
	cfg->show_tips=DFLT_SHOW_TIPS;
	cfg->scanlines=DFLT_SCANLINES;
	return cfg;
}


CFG * ph_get_config_from_script(CFG *cfg, lua_State *ls)
{
	
	lua_getglobal(ls,"scanlines");
	lua_getglobal(ls,"showtips");
	lua_getglobal(ls,"gametype");
	lua_getglobal(ls,"keyright");
	lua_getglobal(ls,"keyleft");
	lua_getglobal(ls,"keydown");
	lua_getglobal(ls,"keyup");
	lua_getglobal(ls,"prefix");
	lua_getglobal(ls,"datadir");
	lua_getglobal(ls,"resolution");
	lua_getglobal(ls,"xres");
	lua_getglobal(ls,"yres");
	lua_getglobal(ls,"fullscreen");
	lua_getglobal(ls,"stretch");
	lua_getglobal(ls,"stretchx");
	lua_getglobal(ls,"stretchy");
	lua_getglobal(ls,"enable_sound");
	lua_getglobal(ls,"sfx");
	lua_getglobal(ls,"music");
	lua_getglobal(ls,"sfxvolume");
	lua_getglobal(ls,"musicvolume");
	lua_getglobal(ls,"controltype");
	cfg->scanlines=(int)lua_tonumber(ls,-22);
	cfg->show_tips=(int)lua_tonumber(ls,-21);
	cfg->game_type=(int)lua_tonumber(ls,-20);
	cfg->key_right=(int)lua_tonumber(ls,-19);
	cfg->key_left=(int)lua_tonumber(ls,-18);
	cfg->key_down=(int)lua_tonumber(ls,-17);
	cfg->key_up=(int)lua_tonumber(ls,-16);
	if (cfg->key_up==0) cfg->key_up=84;
	if (cfg->key_down==0) cfg->key_down=85;
	if (cfg->key_left==0) cfg->key_left=82;
	if (cfg->key_right==0) cfg->key_right=83;
	strcpy(cfg->prefix,lua_tostring(ls,-15));
	strcpy(cfg->datadir,lua_tostring(ls,-14));
	cfg->resolution=(unsigned char)lua_tonumber(ls,-13);
	cfg->xres=(unsigned short int)lua_tonumber(ls,-12);
	cfg->yres=(unsigned short int)lua_tonumber(ls,-11);
	cfg->fullscreen=(unsigned char)lua_tonumber(ls,-10);
	cfg->stretch=(unsigned char)lua_tonumber(ls,-9);
	cfg->stretchx=(unsigned short int)lua_tonumber(ls,-8);
	cfg->stretchy=(unsigned short int)lua_tonumber(ls,-7);
	cfg->enable_sound=(unsigned char)lua_tonumber(ls,-6);
	cfg->sfx=(unsigned char)lua_tonumber(ls,-5);
	cfg->music=(unsigned char)lua_tonumber(ls,-4);
	cfg->sfxvolume=(unsigned char)lua_tonumber(ls,-3);
	cfg->musicvolume=(unsigned char)lua_tonumber(ls,-2);
	cfg->controltype=(unsigned char)lua_tonumber(ls,-1);
	return cfg;
}


CFG * ph_get_config_from_cfg(CFG *cfg, lua_State *ls)
{
int res;

	/* Si el script de configuracion devuelve algun error poner los valores por defecto, si no
	pillar los datos del script. */
	#ifdef WIN32
		res=lua_dofile(ls,DFLT_CONFIG_FILE_W32);
	#else
		res=lua_dofile(ls,DFLT_CONFIG_FILE_UNIX);
	#endif
	if (res!=0)
	{
		ph_log(_("Warning: there's a problem with the configuration file. Assuming defaults.\n"));
		cfg=ph_set_config_defaults(cfg);
	}
	else
	{
		lua_getglobal(ls,"scanlines");
		lua_getglobal(ls,"showtips");
		lua_getglobal(ls,"gametype");
		lua_getglobal(ls,"keyright");
		lua_getglobal(ls,"keyleft");
		lua_getglobal(ls,"keydown");
		lua_getglobal(ls,"keyup");
		lua_getglobal(ls,"prefix");
		lua_getglobal(ls,"datadir");
		lua_getglobal(ls,"resolution");
		lua_getglobal(ls,"xres");
		lua_getglobal(ls,"yres");
		lua_getglobal(ls,"fullscreen");
		lua_getglobal(ls,"stretch");
		lua_getglobal(ls,"stretchx");
		lua_getglobal(ls,"stretchy");
		lua_getglobal(ls,"enable_sound");
		lua_getglobal(ls,"sfx");
		lua_getglobal(ls,"music");
		lua_getglobal(ls,"sfxvolume");
		lua_getglobal(ls,"musicvolume");
		lua_getglobal(ls,"controltype");
		cfg->scanlines=(int)lua_tonumber(ls,-22);
		cfg->show_tips=(int)lua_tonumber(ls,-21);
		cfg->game_type=(int)lua_tonumber(ls,-20);
		cfg->key_right=(int)lua_tonumber(ls,-19);
		cfg->key_left=(int)lua_tonumber(ls,-18);
		cfg->key_down=(int)lua_tonumber(ls,-17);
		cfg->key_up=(int)lua_tonumber(ls,-16);
		strcpy(cfg->prefix,lua_tostring(ls,-15));
		strcpy(cfg->datadir,lua_tostring(ls,-14));
		cfg->resolution=(unsigned char)lua_tonumber(ls,-13);
		cfg->xres=(unsigned short int)lua_tonumber(ls,-12);
		cfg->yres=(unsigned short int)lua_tonumber(ls,-11);
		cfg->fullscreen=(unsigned char)lua_tonumber(ls,-10);
		cfg->stretch=(unsigned char)lua_tonumber(ls,-9);
		cfg->stretchx=(unsigned short int)lua_tonumber(ls,-8);
		cfg->stretchy=(unsigned short int)lua_tonumber(ls,-7);
		cfg->enable_sound=(unsigned char)lua_tonumber(ls,-6);
		cfg->sfx=(unsigned char)lua_tonumber(ls,-5);
		cfg->music=(unsigned char)lua_tonumber(ls,-4);
		cfg->sfxvolume=(unsigned char)lua_tonumber(ls,-3);
		cfg->musicvolume=(unsigned char)lua_tonumber(ls,-2);
		cfg->controltype=(unsigned char)lua_tonumber(ls,-1);
		ph_log(_("Ok.\n"));
	}
	return cfg;
}


char ph_save_config_file(CFG *cfg)
{
FILE *f;
	
	#ifdef WIN32
		f=fopen(DFLT_CONFIG_FILE_W32,"wt");
	#else
		f=fopen(DFLT_CONFIG_FILE_UNIX,"wt");
	#endif
	if (f==NULL) 
	{
		ph_log(_("Warning: Can't write the configuration file, check permission.\n"));
		return 0;
	}
	else
	{
		fprintf(f,_("-- Configuration file for Phantomas PC\n"));	
		fprintf(f,_("-- Lines starting with -- are comments.\n\n"));
		fprintf(f,_("-- Game's resolution, 0 -> 640 x 480, 1 -> 800 x 600, 2 -> 1024 x 768.\n"));
		fprintf(f,"resolution = %d\n",cfg->resolution);
		fprintf(f,_("-- Horizontal resolution of the game, recommended 640.\n"));
		fprintf(f,"xres = %d\n",cfg->xres);
		fprintf(f,_("-- Vertical resolution of the game, recommended 480.\n"));
		fprintf(f,"yres = %d\n",cfg->yres);
		fprintf(f,_("-- Full screen? 0 -> no, 1 -> yes.\n"));
		fprintf(f,"fullscreen = %d\n",cfg->fullscreen);
		fprintf(f,"stretch = %d\n",cfg->stretch);
		fprintf(f,"stretchx = %d\n",cfg->stretchx);
		fprintf(f,"stretchy = %d\n",cfg->stretchy);
		fprintf(f,"enable_sound = %d\n",cfg->enable_sound);
		fprintf(f,"sfx = %d\n",cfg->sfx);
		fprintf(f,"music = %d\n",cfg->music);
		fprintf(f,"sfxvol = %d\n",cfg->sfx);
		fprintf(f,"musicvol = %d\n",cfg->music);
		fprintf(f,"controltype = %d\n",cfg->controltype);
		fprintf(f,"datadir = \"%s\"\n",cfg->datadir);
		fprintf(f,"prefix = \"%s\"\n",cfg->prefix);
		fprintf(f,"keyup = \%d\n",cfg->key_up);
		fprintf(f,"keydown = \%d\n",cfg->key_down);
		fprintf(f,"keyleft = \%d\n",cfg->key_left);
		fprintf(f,"keyright = \%d\n",cfg->key_right);
		fprintf(f,_("-- Type of the game, 0 -> normal (remake), 1 -> classic (speccy).\n"));	
		fprintf(f,"gametype = \%d\n",cfg->game_type);
		fprintf(f,_("-- Show tips? 0 -> no, 1 -> yes.\n"));	
		fprintf(f,"showtips = \%d\n",cfg->show_tips);
		fprintf(f,_("-- Scanlines? 0 -> no, 1 -> yes.\n"));	
		fprintf(f,"scanlines = \%d",cfg->scanlines);
		fclose(f);
	}
	return 1;
}


TEMPCFG * ph_init_temp_configuration(void)
{
TEMPCFG *aux;

	ph_log(_("- Initiating temporal configuration: "));
	aux=(struct TEMPCFG *)malloc(sizeof(struct TEMPCFG));
	if (aux==NULL) 
	{
		ph_log_memory_error("config.c","ph_init_temp_configuration","aux",sizeof(struct TEMPCFG));
		exit(13);
	}
	else
	{
		ph_log(_("Ok.\n"));
	}
	memset(aux,0,sizeof(struct TEMPCFG));

	return aux;
}


void ph_destroy_temp_configuration(TEMPCFG *tcfg)
{
	free(tcfg);
}

