/*
	control.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include "config.h"
#include <allegro.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include "control.h"
#include "control2.h"
#include "log.h"
#include "trans.h"
#include "cfg.h"
#include "texter.h"
#include "menu.h"
#include "help.h"
#include "game.h"
#include "intro.h"
#include "phantom1.h"
#include "sfx.h"



CONTROL * ph_init_control(CFG *cfg)
{
CONTROL *aux;

	ph_log(_("- Initiating control: "));
	
	aux=(struct CONTROL *)malloc(sizeof(struct CONTROL));
	if (aux==NULL) ph_log_memory_error("control.c","ph_init_control","aux",
		sizeof(struct CONTROL));
	memset(aux,0,sizeof(struct CONTROL));
	ph_log(_("Ok.\n"));
	
	
	#if !GP2X
	if (keyboard_needs_poll())
	{
		aux->polling=1;
	}
	#endif
	
	if (!cfg->controltype) aux->type=CTRL_TYPE_KEYBOARD; else aux->type=CTRL_TYPE_JOYSTICK;

	#if GP2X
		cfg->controltype=CTRL_TYPE_JOYSTICK;
	#else
		/* Poner las teclas por defecto */
		aux->key[MOVE_UP]=cfg->key_up;
		aux->key[MOVE_DOWN]=cfg->key_down;
		aux->key[MOVE_LEFT]=cfg->key_left;
		aux->key[MOVE_RIGHT]=cfg->key_right;
		aux->key[MOVE_ACT1]=KEY_Y;
		aux->key[MOVE_ACT2]=KEY_U;
		aux->key[MOVE_ACT3]=KEY_I;
		aux->key[MOVE_ACT4]=KEY_O;
	#endif
			
	return aux;
}


void ph_destroy_control(CONTROL *c)
{
	free(c);
}


CONTROL * ph_read_control(PROGRAM *p, CONTROL *c, TMENU *menu, PHFONT *font)
{
char i;	
static char pressed=0;

	/* Borrar valores viejos */
	c->up=0;
	c->down=0;
	c->left=0;
	c->right=0;
	for (i=0;i<MAXACTIONKEYS;i++) c->action[(int)i]=0;
	
	if (p->pl->inball)
	{
		if (c->polling) poll_keyboard();
		if (key[(int)c->key[MOVE_LEFT]] && !key[(int)c->key[MOVE_RIGHT]] && pressed) 
		{
			p->pl->ballspeed+=6;
			pressed=0;
		}
		if (key[(int)c->key[MOVE_RIGHT]] && !key[(int)c->key[MOVE_LEFT]] && !pressed)
		{
			p->pl->ballspeed+=6;
			pressed=1;
		}
		p->pl->ballspeed--;
		p->pl->x=itofix(p->pl->ballspeed);
		p->pl->aniframe=(fixadd(p->pl->aniframe,itofix(1)));
		if (fixtoi(p->pl->aniframe)>=4) p->pl->aniframe=itofix(0);
		
	}
	else
	{
		switch(c->type)
		{
			case CTRL_TYPE_KEYBOARD:
				if (!p->pl->inchopper || !p->pl->inrocket)
				{
					if (c->polling) poll_keyboard();
					if (key[(int)c->key[MOVE_UP]]) c->up=1;
					if (key[(int)c->key[MOVE_DOWN]]) c->down=1;
					if (key[(int)c->key[MOVE_LEFT]]) c->left=1;
					if (key[(int)c->key[MOVE_RIGHT]]) c->right=1;
					if (key[(int)c->key[MOVE_ACT1]]) c->action[0]=1;
					if (key[(int)c->key[MOVE_ACT2]]) c->action[1]=1;
					if (key[(int)c->key[MOVE_ACT3]]) c->action[2]=1;
					if (key[(int)c->key[MOVE_ACT4]]) c->action[3]=1;
				}
				break;
			
			case CTRL_TYPE_JOYSTICK:
				
				if (!p->pl->inchopper || !p->pl->inrocket || !p->pl->inball)
				{
					poll_joystick();
					if (joy[0].stick[0].axis[0].d1) c->left=1;
					if (joy[0].stick[0].axis[0].d2) c->right=1;
					
					#if GP2X
						if (joy[0].button[4].b) c->left=1;
						if (joy[0].button[5].b) c->right=1;
					#endif
					
					if (joy[0].button[0].b) c->up=1;
					if (joy[0].button[1].b) c->down=1;
				}
				break;
			
			default:
				
				break;
		}
	}
	
	ph_read_common_keys(p, c,menu,font);
	
	return c;
}


CONTROL * ph_init_joystick(CONTROL *c)
{
char res;
	
	ph_log(_("- Initiating joystick: "));
	res=install_joystick(JOY_TYPE_AUTODETECT);
	if (res || !num_joysticks)
	{
		ph_log(_("Error.\n"));
		c->type=CTRL_TYPE_KEYBOARD;
		return c;
	}
	else
	{
		ph_log(_("Ok.\n"));
	}
	c->type=CTRL_TYPE_JOYSTICK;
	return c;
}


void * ph_read_common_keys(PROGRAM *p, CONTROL *c, TMENU *menu, PHFONT *font)
{
int driver;
int olddriver;

	#if GP2X
		poll_joystick();
		if (joy[0].button[5].b && joy[0].button[4].b)
		{
			// Say bye!
			ph_destroy_program(p);
		}
	#endif	
	
	if ((key[KEY_LCONTROL] && key[KEY_Q]) || (key[KEY_RCONTROL] && key[KEY_Q]) || (key[KEY_ALT] && key[KEY_F4]))
	{
		ph_destroy_program(p);
	}

	if (key[KEY_ALT] && key[KEY_ENTER])
	{
		p->cfg->fullscreen^=1;
		if (p->cfg->fullscreen) {olddriver=driver=GFX_AUTODETECT;} else {olddriver=driver=GFX_AUTODETECT_WINDOWED;}
		ph_log(_("- Changing the graphic mode: "));
		
		if (!p->cfg->stretch)
		{
			if (!set_gfx_mode(driver,p->cfg->xres,p->cfg->yres,0,0))
			{
				ph_log(_("Ok.\n"));
			}
			else
			{
				ph_log(_("Error. Returning to previous graphic mode: "));
				if (set_gfx_mode(olddriver,p->cfg->xres,p->cfg->yres,0,0))
				{
					ph_log(_("Error, aborting program...\n"));
				}
				else
				{
					ph_log(_("Ok.\n"));
				}
			}
		}
		else
		{
			if (set_gfx_mode(driver,p->cfg->stretchx,p->cfg->stretchy,0,0))
			{
				set_gfx_mode(olddriver,p->cfg->stretchx,p->cfg->stretchy,0,0);
			}
		}
		
		if (p->cfg->stretch)
		{
			stretch_blit(p->data->dgfx[PHANTGFX_DAT_MISFONDO_BMP].dat,screen,0,0,p->cfg->xres,p->cfg->yres,0,0,p->cfg->stretchx,p->cfg->stretchy);
		}
		else
		{
			blit(p->data->dgfx[PHANTGFX_DAT_MISFONDO_BMP].dat,screen,0,0,0,0,p->cfg->xres,p->cfg->yres);
		}

		if (olddriver==GFX_AUTODETECT_WINDOWED) set_window_title(_("Phantomas PC 0.8"));

	}
	
		if (key[KEY_ESC] && menu->ingame) 
		{
			menu->showmenu^=1;
			p->t->target_cycle=p->t->acycle=0;
			p->menu->activemenu=INGAME_MENU;
			ph_set_menu_options(p->fnt,p->menu);
			rest(100);
		}
		if (joy[0].button[2].b && menu->ingame) 
		{
			menu->showmenu^=1;
			p->t->target_cycle=p->t->acycle=0;
			p->menu->activemenu=INGAME_MENU;
			ph_set_menu_options(p->fnt,p->menu);
			rest(100);
		}
	
	if (menu->showmenu)
	{
			if (key[KEY_UP])
			{
				menu->actelement--;
				if (menu->actelement<0) menu->actelement=menu->elements-1;
			}
		
			if (key[KEY_DOWN])
			{
				menu->actelement++;
				if (menu->actelement>=menu->elements) menu->actelement=0;
			}
	
			if (key[KEY_ENTER])
			{
				menu->activemenu=menu->linknext[(int)menu->actelement];
				menu->actelement=0;
				ph_set_menu_options(font,menu);
				rest(50);
			}
			
			poll_joystick();
			if (joy[0].stick[0].axis[1].d1)
			{
				menu->actelement--;
				if (menu->actelement<0) menu->actelement=menu->elements-1;
			}
		
			if (joy[0].stick[0].axis[1].d2)
			{
				menu->actelement++;
				if (menu->actelement>=menu->elements) menu->actelement=0;
			}
			
			if (joy[0].button[0].b || joy[0].button[1].b)
			{
				menu->activemenu=menu->linknext[(int)menu->actelement];
				menu->actelement=0;
				ph_set_menu_options(font,menu);
				rest(50);
			}
	}
	
	if (key[KEY_F1]) ph_show_help(p);
	
	if (key[KEY_F3]) ph_redefine_keys(p);
	
	if (key[KEY_F10]) ph_abort_game(p);
	
	if (key[KEY_F12]) ph_take_snapshot();
	
	if (key[KEY_F2])
	{
		ph_set_volume(p->gfx,p->snd,p->fnt,p->cfg);
		p->t->acycle=p->t->target_cycle=0;
	}
	
	return NULL;
}


void ph_take_snapshot(void)
{
BITMAP *bmp;
PALETTE pal;
int shots=0;
char *name;
snap_cback cb;
	
	cb=&ph_snapshot_callback;
	shots=for_each_file_ex("snapshot*.bmp",0,0,cb,NULL);
	name=(char *)malloc(sizeof(char)*512);
	memset(name,0,sizeof(char)*512);
	sprintf(name,"snapshot%d.bmp",shots);
	get_palette(pal);
	bmp=screen;
	save_bitmap(name,bmp,pal);
}


int ph_snapshot_callback(const char *filename, int attrib, void *param)
{
	return 0;
}


void ph_redefine_keys(PROGRAM *p)
{
BITMAP *bmp;
int i=0;
const char *action[]={_("High jump"),_("Long jump"),_("Left"),_("Right")};
char act=0;
char redkey[4];

	bmp=create_bitmap(p->gfx->virt[0]->w,p->gfx->virt[0]->h);
	clear_to_color(bmp,makecol(0,0,0));
	set_trans_blender(200,200,200,200);
	drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
	rectfill(p->gfx->virt[0],0,81,479,249,makecol(0,0,50));
	set_trans_blender(255,255,255,255);
	drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
	draw_sprite(p->gfx->virt[0],p->gfx->smalllogo,(480>>1)-(p->gfx->smalllogo->w>>1),20);
	ph_set_text_properties(p->fnt,makecol(255,255,0),-1,TXT_TITLE,TXT_CENTER,20,0,1);
	ph_shprintf(p->gfx->virt[0],p->fnt,240,80,_("Redefine keys"));
	clear_keybuf();
	blit(p->gfx->virt[0],bmp,0,0,0,0,bmp->w,bmp->h);
	
	while(act<4)
	{
		draw_sprite(p->gfx->virt[0],bmp,0,0);
		ph_set_text_properties(p->fnt,makecol(255,255,255),-1,TXT_TITLE,TXT_CENTER,20,0,1);
		ph_shprintf(p->gfx->virt[0],p->fnt,240,110+40,_("Press a key for %s"),action[(int)act]);
		
		ph_draw(p->gfx,p->cfg);
		
		clear_keybuf();
		while (!keypressed())
		{
			#if HAVE_ALDUMB_H
				if (p->cfg->music && p->cfg->enable_sound) al_poll_duh(p->snd->music->mpl);
			#endif
		}
		redkey[(int)act]=readkey()>>8;
		clear_keybuf();
		
		ph_shprintf(p->gfx->virt[0],p->fnt,240,130+40,"%s",scancode_to_name(redkey[(int)act]));
		ph_draw(p->gfx,p->cfg);
		rest(100);
		act++;
	}
	
	for (i=0;i<4;i++)
	{
		p->c->key[(int)i]=redkey[(int)i];
	}
	
	destroy_bitmap(bmp);
	p->cfg->key_up=p->c->key[0];
	p->cfg->key_down=p->c->key[1];
	p->cfg->key_left=p->c->key[2];
	p->cfg->key_right=p->c->key[3];
	p->cfg->controltype=0;
	p->c->type=CTRL_TYPE_KEYBOARD;
	p->t->acycle=p->t->target_cycle=0;
}


void ph_abort_game(PROGRAM *p)
{
char *res[2]={_("Yes"),_("No")};
char ok;
char cancel;
char restart=0;	
	set_trans_blender(200,200,200,200);
	drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
	rectfill(p->gfx->virt[0],0,81,479,249,makecol(0,0,50));
	set_trans_blender(255,255,255,255);
	drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
	draw_sprite(p->gfx->virt[0],p->gfx->smalllogo,(480>>1)-(p->gfx->smalllogo->w>>1),20);
	ph_set_text_properties(p->fnt,makecol(255,255,0),-1,TXT_TITLE,TXT_CENTER,20,0,1);
	ph_shprintf(p->gfx->virt[0],p->fnt,240,100+40,_("Abort the game"));
	ph_set_text_properties(p->fnt,makecol(255,255,255),-1,TXT_TITLE,TXT_CENTER,15,0,1);
	ph_shprintf(p->gfx->virt[0],p->fnt,240,130+40,_("Are you sure you want to restart the game? %s/%s"),res[0],res[1]);
	ph_draw(p->gfx,p->cfg);
	ok=ph_ascii_to_scancode(*res[0]);
	cancel=ph_ascii_to_scancode(*res[1]);
	
	while(1)
	{
		if (key[(int)cancel] || key[KEY_ESC]) break;
		
		if (key[(int)ok])
		{
			ph_game_over(p);
			restart=1;
			break;
		}
		
		#if HAVE_ALDUMB_H
			if (p->cfg->music && p->cfg->enable_sound) al_poll_duh(p->snd->music->mpl);
		#endif
	}
	
	if (restart) 
	{
		#if HAVE_ALDUMB_H
			if (p->cfg->music && p->cfg->enable_sound) al_stop_duh(p->snd->music->mpl);
		#endif
		ph_start_game(p);
	}
	p->t->acycle=p->t->target_cycle=0;
}


char ph_ascii_to_scancode(char ascii)
{
unsigned char res;

	ascii=tolower(ascii);
	res=ascii-96;
	return res;
}


