/*
	data.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include "config.h"
#include <allegro.h>

#if HAVE_DUMB_H
	#include <dumb.h>
#endif

#if HAVE_ALDUMB_H
	#include <aldumb.h>
#endif

#include <string.h>
#include "data.h"
#include "trans.h"
#include "log.h"
#include "path.h"
#include "phantom1.h"
#include "loadsc.h"

DATA * ph_init_data(CFG *cfg, char *path)
{
DATA *aux;
char *buf;
char *dfile={"phantom1.dat"};
DATAFILE *tmp1;
DATAFILE *tmp2;
DATAFILE *tmp3;
//DATAFILE *tmp4;

	ph_log(_("- Initiating data: "));
	
	aux=(struct DATA *)malloc(sizeof(struct DATA));
	if (aux==NULL) ph_log_memory_error("data.c","ph_init_data","aux",
		sizeof(struct DATA));
	memset(aux,0,sizeof(struct DATA));
	
	buf=(char *)malloc(sizeof(char)*512);
	if (buf==NULL) ph_log_memory_error("data.c","ph_init_data","buf",
		sizeof(char)*512);
	
	ph_log(_("Ok.\n"));
		
	memset(buf,0,sizeof(char)*512);
	strcpy(buf,cfg->datadir);
	strcat(buf,dfile);
	ph_log(_("- Loading data file %s: "),ph_get_path(cfg,path,buf));
	/* Load parent data file */
	aux->df=load_datafile_callback(ph_get_path(cfg,path,buf),ph_load_callback);
	if (aux->df==NULL)
	{
		ph_log(_("Fatal error: Can't load the file: %s."),ph_get_path(cfg,path,buf));
		exit(13);
	}
	else
	{
		ph_log(_("Ok.\n"));
	}

	/* Load children data files */
	tmp1=(DATAFILE *)find_datafile_object(aux->df,"PHANTGFX_DAT");
	tmp2=(DATAFILE *)find_datafile_object(aux->df,"PHANTSND_DAT");
	tmp3=(DATAFILE *)find_datafile_object(aux->df,"PHANTFNT_DAT");
	//tmp4=(DATAFILE *)find_datafile_object(aux->df,"PHANTSTP_DAT");
	aux->dgfx=(DATAFILE *)tmp1->dat;
	aux->dsnd=(DATAFILE *)tmp2->dat;
	aux->dfnt=(DATAFILE *)tmp3->dat;
	//aux->dstp=(DATAFILE *)tmp4->dat;
	free(buf);
	return aux;
}


void ph_destroy_data(DATA *data)
{
	unload_datafile(data->df);
	free(data);
}


void ph_assign_data(DATA *data, CFG *cfg, GFX *gfx, SOUND *snd, PHFONT *fnt, char *path)
{
int i;
char *buf;
#if HAVE_ALOGG_H
	DATAFILE *dogg;
#endif

	buf=(char *)malloc(sizeof(char)*512);
	memset(buf,0,sizeof(char)*512);
	
	for (i=PHANTGFX_DAT_BACK000_BMP;i<PHANTGFX_DAT_DEC00_0_BMP;i++)
	{
		gfx->backspr[i-PHANTGFX_DAT_BACK000_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	for (i=PHANTGFX_DAT_DEC00_0_BMP;i<PHANTGFX_DAT_ENE00_0_0_BMP-1;i++)
	{
		gfx->decspr[i-PHANTGFX_DAT_DEC00_0_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	for (i=PHANTGFX_DAT_PHA00_0_1_BMP;i<PHANTGFX_DAT_PLA00_0_BMP;i++)
	{
		gfx->phspr[i-PHANTGFX_DAT_PHA00_0_1_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	for (i=PHANTGFX_DAT_PLA00_0_BMP;i<PHANTGFX_DAT_X0_BMP-1;i++)
	{
		gfx->plaspr[i-PHANTGFX_DAT_PLA00_0_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	
	gfx->traperlogo[1]=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCTR2_BMP].dat;
	gfx->traperlogo[2]=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCTR3_BMP].dat;
	gfx->traperlogo[3]=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCTR4_BMP].dat;
	gfx->traperlogo[4]=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCTR5_BMP].dat;
	gfx->smalllogo=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISLOGO_BMP].dat;
	gfx->bigphan=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCBIGPHANT_BMP].dat;
	gfx->logo=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISLOGO1_BMP].dat;
	gfx->face=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISFACE_BMP].dat;
	gfx->energy=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISENERGY_BMP].dat;
	gfx->gameover=(BITMAP *)data->dgfx[PHANTGFX_DAT_MISCGAMEOVER_BMP].dat;
	gfx->spr=(BITMAP *)data->dgfx[PHANTGFX_DAT_DEC06_0_BMP].dat;
	
	for (i=PHANTGFX_DAT_LIFE00_0_BMP;i<=PHANTGFX_DAT_LIFE00_3_BMP;i++)
	{
		gfx->lifespr[i-PHANTGFX_DAT_LIFE00_0_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	for (i=PHANTGFX_DAT_SW00_0_BMP;i<=PHANTGFX_DAT_SW01_3_BMP;i++)
	{
		gfx->swspr[i-PHANTGFX_DAT_SW00_0_BMP]=(BITMAP *)data->dgfx[i].dat;
	}
	
	#if HAVE_DUMB_H
	if (cfg->music && cfg->enable_sound)
	{
		ph_log(_("- Loading MOD music: "));
		snd->music->duh=(DUH *)data->dsnd[PHANTSND_DAT_MENU_XM].dat;
		if (snd->music->duh==NULL) 
		{
			ph_log(_("Error.\n"));
			cfg->music=0;
		}
		else
		{
			ph_log(_("Ok.\n"));
			cfg->music=1;
			dumb_resampling_quality=4;
		}
	}
	#endif
	
	#if HAVE_ALOGG_H
		dogg=find_datafile_object(data->df,"SONG3_OGG");
		ASSERT(dogg!=NULL);
		if (cfg->music && cfg->enable_sound)
		{
			snd->music->ogg=alogg_create_sample(dogg);
		}
	#endif

	if (cfg->enable_sound && cfg->sfx)
	{

		snd->sfx->push=(SAMPLE *)data->dsnd[PHANTSND_DAT_PUSH_WAV].dat;
		snd->sfx->hjump=(SAMPLE *)data->dsnd[PHANTSND_DAT_BOING_WAV].dat;
		snd->sfx->ljump=(SAMPLE *)data->dsnd[PHANTSND_DAT_BOING_WAV].dat;
		snd->sfx->hit=(SAMPLE *)data->dsnd[PHANTSND_DAT_HIT_WAV].dat;
		snd->sfx->eat=(SAMPLE *)data->dsnd[PHANTSND_DAT_EAT_WAV].dat;
		snd->sfx->walk=(SAMPLE *)data->dsnd[PHANTSND_DAT_WALK_WAV].dat;

/*		snd->sfx->push=NULL;
		snd->sfx->hjump=NULL;
		snd->sfx->ljump=NULL;
		snd->sfx->hit=NULL;
		snd->sfx->eat=NULL;
		snd->sfx->walk=NULL;*/
	}
	else
	{
		/*snd->sfx->push=NULL;
		snd->sfx->hjump=NULL;
		snd->sfx->ljump=NULL;
		snd->sfx->hit=NULL;
		snd->sfx->eat=NULL;
		snd->sfx->walk=NULL;*/
	}
	
	#if HAVE_ALFONT_H
		fnt->ufnt=alfont_load_font_from_mem((char *)data->dfnt[PHANTFNT_DAT_VERA_TTF].dat, data->dfnt[PHANTFNT_DAT_VERA_TTF].size);
		fnt->fnt=alfont_load_font_from_mem((char *)data->dfnt[PHANTFNT_DAT_VERABD_TTF].dat, data->dfnt[PHANTFNT_DAT_VERABD_TTF].size);
	#else
		fnt->ufnt=font;
		fnt->fnt=font;
	#endif
	
	free(buf);

}


void ph_load_callback(DATAFILE *dat_obj)
{
static unsigned long int a=0L;
static char firsttime=1;
static BITMAP *v,*load,*virt;
static int col1,col2,i=0;
static float p;

	if (firsttime)
	{
		load=(BITMAP *)data[EXAMPLE].dat;
		v=create_bitmap(load->w,load->h);
		virt=create_bitmap(640,480);
		col1=makecol(200,200,0);
		col2=makecol(150,150,0);
		p=(load->w-40);
		p/=100;
	}
	a++;
	firsttime=0;
	clear_to_color(v,makecol(0,0,0));
	draw_sprite(v,load,0,0);
	i=((a*100)/442)*(int)p;
	line(v,35,32,35+i,32,col1);
	line(v,35,33,35+i,33,col2);
	line(v,35,34,35+i,34,col2);
	line(v,35,35,35+i,35,col2);
	line(v,35,36,35+i,36,col2);
	line(v,35,37,35+i,37,col1);
	
	draw_sprite(virt,v,180,147);
	
	#if GP2X
		stretch_blit(virt,screen,0,0,640,480,0,0,320,240);
	#else
		blit(virt,screen,0,0,0,0,640,480);
	#endif
	
	if (a==442)
	{
		destroy_bitmap(v);
		destroy_bitmap(virt);
	}
}
