/*
	enemy.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include "enemy.h"
#include "phantom1.h"
#include "trans.h"
#include "ppcd.h"
#include "blood.h"


ENEMY * ph_init_enemies(void)
{
ENEMY *aux;
AUXENE *auxene;

	ph_log(_("- Initiating enemies: "));
	aux=(struct ENEMY *)malloc(sizeof(struct ENEMY));
	if (aux==NULL) ph_log_memory_error("enemy.c","ph_init_enemies","aux",
		sizeof(struct ENEMY));
	memset(aux,0,sizeof(struct ENEMY));
	auxene=(struct AUXENE *)malloc(sizeof(struct AUXENE));
	if (auxene==NULL) ph_log_memory_error("enemy.c","ph_init_enemies","auxene",
		sizeof(struct AUXENE));
	memset(auxene,0,sizeof(struct AUXENE));
	ph_log(_("Ok.\n"));
	aux->auxene=auxene;
	return aux;
}

void ph_destroy_enemies(ENEMY * ene)
{
	free(ene->auxene);
	free(ene);
}

/*
create_enemy(type,x,y,xlimitl,xlimitr,ylimitu,ylimitd,movtype,aniframe,anirate,right,xspeed,yspeed,power)
*/
void ph_create_enemy(ENEMY *enemy, GFX *gfx, DATA *data)
{
int a;
char move_types[ENEMY_NUMBER];

	memset(move_types,EMOVE_NORMAL,sizeof(char)*ENEMY_NUMBER);
	move_types[3]=EMOVE_RIGHT;move_types[16]=EMOVE_RIGHT;move_types[19]=EMOVE_RIGHT;
	move_types[20]=EMOVE_RIGHT;move_types[28]=EMOVE_RIGHT;move_types[32]=EMOVE_RIGHT;
	move_types[11]=EMOVE_NONE;move_types[14]=EMOVE_NONE;move_types[15]=EMOVE_NONE;
	move_types[21]=EMOVE_NONE;move_types[23]=EMOVE_NONE;move_types[26]=EMOVE_NONE;
	
	enemy->power[(int)enemy->actenemy]=enemy->auxene->power;
	enemy->framespeed[(int)enemy->actenemy]=enemy->auxene->anirate;
	enemy->actframe[(int)enemy->actenemy]=enemy->auxene->aniframe;
	enemy->active[(int)enemy->actenemy]=1;
	enemy->y1limit[(int)enemy->actenemy]=enemy->auxene->ylimitu;
	enemy->y2limit[(int)enemy->actenemy]=enemy->auxene->ylimitd;
	enemy->xspeed[(int)enemy->actenemy]=enemy->auxene->xspeed;
	enemy->yspeed[(int)enemy->actenemy]=enemy->auxene->yspeed;
	enemy->type[(int)enemy->actenemy]=enemy->auxene->type;
	enemy->up[(int)enemy->actenemy]=0;
	enemy->right[(int)enemy->actenemy]=enemy->auxene->right;
	enemy->x1limit[(int)enemy->actenemy]=enemy->auxene->xlimitl;
	enemy->x2limit[(int)enemy->actenemy]=enemy->auxene->xlimitr;
	enemy->x[(int)enemy->actenemy]=enemy->auxene->x;
	enemy->y[(int)enemy->actenemy]=enemy->auxene->y;
	enemy->mov_type[(int)enemy->actenemy]=move_types[(int)enemy->type[(int)enemy->actenemy]];

	for (a=0;a<4;a++)
	{
		gfx->enespr[(int)enemy->actenemy][(int)a]=(BITMAP *)data->dgfx[PHANTGFX_DAT_ENE00_0_0_BMP+((enemy->type[(int)enemy->actenemy]<<2)+a)].dat;
	}
	
	enemy->actenemy++;
	if (enemy->actenemy>=MAX_ENEMIES) enemy->actenemy=0;
}


void ph_move_enemies(ENEMY *enemy)
{
int i;
int aux;

	for (i=0;i<MAX_ENEMIES;i++)
	{
		if (enemy->active[(int)i])
		{
			aux=enemy->actenemy;
			//enemy->y[(int)i]+=enemy->yspeed[(int)i];
			enemy->actframe[(int)i]++;
			if (enemy->actframe[(int)i]==enemy->framespeed[(int)i]<<2) enemy->actframe[(int)i]=0;
			
			if (enemy->mov_type[i]!=EMOVE_NONE)
			{
				if (enemy->x[i]<=enemy->x1limit[i] && !enemy->right[i])
				{
					enemy->right[i]=1;
				}
				
				if (enemy->x[i]>=enemy->x2limit[i] && enemy->right[i])
				{
					enemy->right[i]=0;
				}
				
				if (enemy->x[i]>=enemy->x1limit[i] && !enemy->right[i])
				{
					enemy->x[i]-=enemy->xspeed[i];
				}
				
				if (enemy->x[i]<=enemy->x2limit[i] && enemy->right[i])
				{
					enemy->x[i]+=enemy->xspeed[i];
				}
				
				if (enemy->y[i]<=enemy->y1limit[i] && enemy->up[i])
				{
					enemy->up[i]=0;
				}
				
				if (enemy->y[i]<enemy->y2limit[i] && !enemy->up[i])
				{
					enemy->y[i]+=enemy->yspeed[i];
				}
				
				if (enemy->y[i]>=enemy->y2limit[i] && !enemy->up[i])
				{
					enemy->up[i]=1;
				}
			
				if (enemy->y[i]>enemy->y1limit[i] && enemy->up[i])
				{
					enemy->y[i]-=enemy->yspeed[i];
				}
				
			}
			/*
			if (enemy->y[(int)i]>=480) 
			{
				enemy->active[(int)i]=0;
				enemy->actenemy++;
				if (enemy->actenemy>=PH_MAX_ENEMIES) enemy->actenemy=0;
			}*/
		}
	}
}


void ph_draw_enemies(ENEMY *enemy, GFX *gfx)
{
int i;

	for (i=0;i<MAX_ENEMIES;i++)
	{
		if (enemy->active[(int)i])
		{
			if (enemy->mov_type[(int)i]==EMOVE_RIGHT && !enemy->right[(int)i])
			{
				draw_sprite_h_flip(gfx->virt[0],gfx->enespr[(int)i][enemy->actframe[(int)i]/enemy->framespeed[(int)i]],enemy->x[(int)i],enemy->y[(int)i]);
			}
			else
			{
				draw_sprite(gfx->virt[0],gfx->enespr[(int)i][enemy->actframe[(int)i]/enemy->framespeed[(int)i]],enemy->x[(int)i],enemy->y[(int)i]);
			}
		}
		
	}
}


void ph_damage_player(ENEMY *ene, PLAYER *pl, GFX *gfx, BLOODSPOT *bs, SOUND *snd, CFG *cfg)
{
char i;
BITMAP *aux=NULL;
BITMAP *aux1=NULL;

	clear(gfx->phhelper);
	aux1=gfx->phhelper;
	
	if (pl->falling)
	{
		aux=gfx->phspr[5+fixtoi(pl->aniframe)];
		if (!pl->right) blit(aux,aux,0,0,0,0,aux->w,aux->h);
	}
	if (pl->jumping)
	{
		aux=gfx->phspr[4];
		if (!pl->right) blit(aux,aux,0,0,0,0,aux->w,aux->h);
	}
	if (!pl->jumping && !pl->falling)
	{
		aux=gfx->phspr[fixtoi(pl->aniframe)];
	}
	if (!pl->right) 
	{
		draw_sprite_h_flip(aux1,aux,0,0);
		aux=aux1;
	}

	for (i=0;i<MAX_ENEMIES;i++)
	{
		if (ene->active[(int)i])
		{
			if (sn_collision(aux,fixtoi(pl->x),fixtoi(pl->y),gfx->enespr[(int)i][ene->actframe[(int)i]/ene->framespeed[(int)i]],ene->x[(int)i],ene->y[(int)i],pl->gametype))
			{
				//ph_create_bloodspot(bs,100,fixtoi(pl->x),fixtoi(pl->y));
				pl->energy-=2;
				pl->damaging=1;
				if (cfg->enable_sound && cfg->sfx) ph_play_sample(snd->sfx->hit,fixtoi(pl->x),cfg);
				if (!pl->firsttimehit) pl->firsttimehit=1;
			}
		}
	}
}
