/*
	gfx.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

/*
TODO: Que funcione lo de escalar la pantalla.
*/

#include "config.h"
#include <string.h>
#include "gfx.h"
#include "log.h"
#include "trans.h"

GFX * ph_init_gfx(CFG *cfg)
{
struct virtdim
{
	int x[MAXVIRTSCREENS];
	int y[MAXVIRTSCREENS];
};

int driver;
const char * drivermsg[]={_("windowed"),_("at fullscreen")};
GFX *aux;
int i;
struct virtdim virtdim=
	{
		{480,240,16,16,512,512,640},
		{288,16,288,288,16,16,480}
	};

	aux=(struct GFX *)malloc(sizeof(struct GFX));
	if (aux==NULL)
	{
		ph_log_memory_error("gfx.c","ph_init_gfx","aux",sizeof(struct GFX));
	}
	memset(aux,0,sizeof(struct GFX));

	#if !GP2X
		if (!cfg->stretch)
			ph_log(_("- Initiating graphic mode (%d x %d @ 16 bpp, running %s): "),cfg->xres, cfg->yres, drivermsg[cfg->fullscreen]);
		else
			ph_log(_("- Initiating graphic mode (%d x %d @ 16 bpp, running %s): "),cfg->stretchx, cfg->stretchy, drivermsg[cfg->fullscreen]);
	#else
			cfg->stretch=1;
			cfg->xres=640;
			cfg->yres=480;
			cfg->stretchx=320;
			cfg->stretchy=240;
			cfg->fullscreen=1;
			ph_log(_("- Initiating graphic mode (%d x %d @ 16 bpp, running %s): "),cfg->stretchx, cfg->stretchy, drivermsg[cfg->fullscreen]);
	#endif
	set_color_depth(16);
	
	if (cfg->fullscreen) {driver=GFX_AUTODETECT;} else {driver=GFX_AUTODETECT_WINDOWED;}
	
	if (!cfg->stretch)
	{
		if (set_gfx_mode(driver,cfg->xres,cfg->yres,0,0))
		{
			ph_log(_("Error: Can't set the graphic mode.\n"));
			if (driver==GFX_AUTODETECT)
			{
				ph_log(_("- Trying windowed mode: "));
				if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,cfg->xres,cfg->yres,0,0))
				{
					ph_log(_("Fatal error.\n"));
					exit(13);
				}
			}
			if (driver==GFX_AUTODETECT_WINDOWED)
			{
				ph_log(_("- Trying fullscreen mode: "));
				if (set_gfx_mode(GFX_AUTODETECT,cfg->xres,cfg->yres,0,0))
				{
					ph_log(_("Fatal error.\n"));
					exit(13);
				}
			}
		}
	}
	else
	{
		if (set_gfx_mode(driver,cfg->stretchx,cfg->stretchy,0,0))
		{
			ph_log(_("Error: Can't set the graphic mode.\n"));
			if (driver==GFX_AUTODETECT)
			{
				ph_log(_("- Trying windowed mode: "));
				if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,cfg->xres,cfg->yres,0,0))
				{
					ph_log(_("Fatal error.\n"));
					exit(13);
				}
			}
			if (driver==GFX_AUTODETECT_WINDOWED)
			{
				ph_log(_("- Trying fullscreen mode: "));
				if (set_gfx_mode(GFX_AUTODETECT,cfg->xres,cfg->yres,0,0))
				{
					ph_log(_("Fatal error.\n"));
					exit(13);
				}
			}
		}
	}
	
	if (driver==GFX_AUTODETECT_WINDOWED) set_window_title(_("Phantomas PC 0.8"));
	
	ph_log(_("Ok.\n"));
	ph_log(_("- Initiating the virtual screens (%d x %d): "),cfg->xres, cfg->yres);
	
	for (i=0;i<MAXVIRTSCREENS;i++)
	{
		aux->virt[i]=create_bitmap(virtdim.x[i], virtdim.y[i]);
		if (aux->virt[i]==NULL)
		{
			ph_log(_("Fatal error: Can't create the virtual screens.\n"));
			exit(13);
		}
		clear_to_color(aux->virt[i],makecol(0,0,0));
	}
	ph_log(_("Ok.\n"));
	
	if (cfg->stretch)
	{
		ph_log(_("- Initiating the virtual screen for scaling: "));
		aux->svirt=create_bitmap(640,480);
		if (aux->svirt==NULL)
		{
			ph_log(_("Error: Can't create the virtual screen to scale the game.\n"));
		}
		else
			ph_log(_("Ok.\n"));
		clear_to_color(aux->svirt,makecol(0,0,0));
	}
	
	ph_log(_("- Initiating phantomas helper: "));
	aux->phhelper=create_bitmap(32,32);
	if (aux->phhelper==NULL)
	{
		ph_log(_("Error: Can't create the phantomas helper sprite.\n"));
	}
	ph_log(_("Ok.\n"));
	clear_to_color(aux->phhelper,makecol(0,0,0));
	
	return aux;
}


void ph_destroy_gfx(CFG *cfg, GFX *gfx)
{
int i;
	
	if (cfg->stretch) destroy_bitmap(gfx->svirt);
	destroy_bitmap(gfx->phhelper);
	for (i=0;i<MAXVIRTSCREENS;i++)
	{
		destroy_bitmap(gfx->virt[i]);
	}
	free(gfx);
}


void ph_draw(GFX *gfx, CFG *cfg)
{
int i;

	acquire_screen();
	vsync();
	if (!cfg->stretch)
	{
		if (cfg->scanlines)
		{
			set_trans_blender(120,120,120,120);
			drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
			for (i=0;i<480;i+=2)
			{
				hline(gfx->virt[0],0,i,480,makecol(10,10,10));
			}
			set_trans_blender(255,255,255,255);
			drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
		}
		blit(gfx->virt[0],screen,0,0,80,76,cfg->xres,cfg->yres);
		blit(gfx->virt[1],screen,0,0,320,390,240,16);
	}
	else
	{
		stretch_blit(gfx->virt[1],screen,0,0,gfx->virt[1]->w,gfx->virt[1]->h,160,200,120,8);
		stretch_blit(gfx->virt[0],screen,0,0,480,288,40,38,240,144);
	}
	release_screen();
}
