 /*
	gllist.c
	Description: Una lista enlazada de cadenas
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/


#include <stdio.h>
#include <string.h>
#include "gllist.h"
#include "log.h"
#include "trans.h"


/* Esta funcion inicia la lista y devuelve un puntero al primer elemento de la lista */
LIST * ph_init_list(void)
{
LIST * aux;

	aux=(struct LIST *)malloc(sizeof(struct LIST));
	memset(aux,0,sizeof(struct LIST));
	aux->id=0;
	aux->data=NULL;
	aux->next=NULL;
	return aux;
}


/* Esta funcion anade un elemento a la lista enlazada. */
void ph_add_element(LIST *lh, char *data)
{
LIST *laux;
LIST *list;
static int id=1;


	/* Reservar memoria para otro elemento */
	list=(struct LIST *)malloc(sizeof(struct LIST));
	memset(list,0,sizeof(struct LIST));

	list->data=(char *)malloc(sizeof(char)*(strlen(data)+1));
	memset(list->data,0,sizeof(char)*(strlen(data)+1));
	
	/* Lista auxiliar apunta al inicio de la lista */
	laux=lh;

	/* Mientras que el nodo siguiente sea != NULL avanzamos por la lista */
	while(laux->next!=NULL)
		laux=laux->next;

	/* Poner el identificador del elemento y sumar 1 al identificador para dejarlo
	preparado para el siguiente elemento */
	list->id=id;
	id++;

	/* Copiar la cadena */
	strcpy(list->data,data);
	
	/* Poner el siguiente elemento de la lista a nulo ya que no hay nada */
	list->next=NULL;
	laux->next=list;
}


/* Esta funcion libera TODA la memoria utilizada por la lista enlazada. */
void ph_destroy_list(LIST *pl)
{
LIST *aux;

	aux=pl;

	while(pl!=NULL)
	{
		aux=pl;
		pl=pl->next;
		free(aux->data);
		free(aux);
	}
}


/* Simple funcion para mostrar el contenido de la lista */
void ph_view_list(LIST *lh)
{
LIST *aux;

	aux=lh;

	while(aux!=NULL)
	{
		fprintf(stdout,"Id: %d  String: %s\n",aux->id,(char *)aux->data);
		aux=aux->next;
	}
}

/* Esta funcion elimina un elemento de la lista; este elemento viene dado por el parametro id, devuelve un entero dependiendo de lo siguiente:
0 -> Si el elemento se ha podido borrar.
1 -> Si no se ha encontrado el elemento.
2 -> Si es el principio de la lista. */
int ph_remove_element(LIST *lh, int id)
{
LIST *aux;
LIST *aux1;
LIST *aux2;

	/* La cabeza de la lista no se puede eliminar a no ser que sea con la funcion
	DestroyList */
	if (lh->next==NULL || id==0) return 2;

	/* Hacer que los punteros auxiliares apunten al principio de la lista */
	aux=aux1=lh;

	while(aux->next!=NULL)
	{
		aux1=aux;
		aux=aux->next;

		/* Si se encuentra el identificador se borra el elemento y devolvemos 0 */
		if (aux->id==id)
		{
			aux1->next=aux->next;
			aux2=aux;
			free(aux2->data);
			free(aux2);
			return 0;
		}
	}
	/* No se ha encontrado el identificador */
	return 1;
}


/* Esta funcion devuelve un puntero al elemento con el identificador id, o NULL si no se encuentra. */
LIST * ph_get_element(LIST *lh, int id)
{
LIST *aux;

	aux=lh;

	while(aux!=NULL)
	{
		if (aux->id==id) return aux;
		aux=aux->next;
	}
	return NULL;
}


