/*
	sfx.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include <allegro.h>
#include <malloc.h>
#include <string.h>
#include "config.h"

#if HAVE_ALOGG_H
	#include <alogg/alogg.h>
#endif

#if HAVE_DUMB_H
	#include <dumb.h>
#endif

#if HAVE_ALDUMB_H
	#include <aldumb.h>
#endif

#include "cfg.h"
#include "log.h"
#include "sfx.h"
#include "trans.h"

SOUND * ph_init_sound(CFG *cfg)
{
SOUND *aux;

	if (!cfg->enable_sound)
	{
		ph_log(_("- Sound disabled.\n"));
		return NULL;
	}
	
	else
	{
		ph_log(_("- Initiating sound system: "));
		aux=(struct SOUND *)malloc(sizeof(struct SOUND));
		if (aux==NULL) ph_log_memory_error("sfx.c","ph_init_sound","aux",sizeof(struct SOUND));
		memset(aux,0,sizeof(struct SOUND));
		//reserve_voices(16,0);
		if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) != 0) 
		{
			ph_log(_("Error initiating sound system.\n"));
			cfg->sfx=0;
			cfg->music=0;
			cfg->enable_sound=0;
		}
		else
		{
			ph_log(_("Ok.\n"));
//			fprintf(stdout,"\n- %d\n",get_mixer_voices());
		}
		
		ph_log(_("- Initiating sound effects: "));
		aux->sfx=ph_init_sfx();
		
		if (aux->sfx!=NULL)
		{
			if (cfg->sfx) 
			{
				ph_log(_("Ok.\n"));
			}
			else
			{
				ph_log(_("Sound effects disabled.\n"));
			}
		}
		
		ph_log(_("- Initiating music: "));
		aux->music=ph_init_music();
		if (aux->music!=NULL)
		{
			if (cfg->music) 
			{
				ph_log(_("Ok.\n"));
			}
			else
			{
				ph_log(_("Music disabled.\n"));
			}
		}
}	
	return aux;
}


void ph_destroy_sound(CFG *cfg, SOUND *sound)
{
#if HAVE_ALDUMB_H
//	if (cfg->music && cfg->enable_sound)
		//al_stop_duh(sound->music->mpl);
#endif

#if HAVE_ALOGG_H
	if (cfg->music && cfg->enable_sound)
	{
		destroy_sample(sound->music->ogg);
	}
	alogg_exit();
#endif

	if (cfg->enable_sound)
	{
		free(sound->sfx);
		free(sound->music);
		free(sound);
	}
}


SFX * ph_init_sfx(void)
{
SFX *aux;

	aux=(struct SFX *)malloc(sizeof(struct SFX));
	if (aux==NULL) ph_log_memory_error("sfx.c","ph_init_sfx","aux",sizeof(struct SFX));
	return aux;
}


MUSIC * ph_init_music(void)
{
MUSIC *aux;

	aux=(struct MUSIC *)malloc(sizeof(struct MUSIC));
	if (aux==NULL) ph_log_memory_error("sfx.c","ph_init_music","aux",sizeof(struct MUSIC));
	#if HAVE_DUMB_H
		atexit(&dumb_exit);
		dumb_register_packfiles();
		dumb_register_dat_xm(DAT_ID('X','M',' ',' '));
	#endif
	
	#if HAVE_ALOGG_H
		alogg_init();
	#endif
	return aux;
}


void ph_play_sample(SAMPLE *sample, signed short int x, CFG *cfg)
{
int pan;
	
	if (sample==NULL) return;
	if (cfg->enable_sound && cfg->sfx)
	{
		pan=(255*x)/cfg->xres;
		play_sample(sample,255,pan,1000,0);
	}
}


void ph_set_volume(GFX *gfx, SOUND *snd, PHFONT *fnt, CFG *cfg)
{
BITMAP *bmp;
static int svol=0;
#if HAVE_ALDUMB_H
	float aux=0;
#endif
	
	svol=(int)cfg->sfxvolume;
	bmp=create_bitmap(gfx->virt[0]->w,gfx->virt[0]->h);
	if (!bmp) return;
	clear_to_color(bmp,makecol(0,0,0));
	set_trans_blender(200,200,200,200);
	drawing_mode(DRAW_MODE_TRANS,NULL,0,0);
	rectfill(gfx->virt[0],0,81,479,249,makecol(0,0,50));
	set_trans_blender(255,255,255,255);
	drawing_mode(DRAW_MODE_SOLID,NULL,0,0);
	draw_sprite(gfx->virt[0],gfx->smalllogo,(480>>1)-(gfx->smalllogo->w>>1),20);
	ph_set_text_properties(fnt,makecol(255,255,0),-1,TXT_TITLE,TXT_CENTER,20,0,1);
	ph_shprintf(gfx->virt[0],fnt,240,80,_("Set volume"));
	clear_keybuf();
	blit(gfx->virt[0],bmp,0,0,0,0,bmp->w,bmp->h);
	
	while(!key[KEY_ESC])
	{
		draw_sprite(gfx->virt[0],bmp,0,0);
		ph_set_text_properties(fnt,makecol(255,255,255),-1,TXT_TITLE,TXT_CENTER,20,0,1);
		ph_shprintf(gfx->virt[0],fnt,240,150,_("Sound volume (10-100): %d"),svol);
		ph_shprintf(gfx->virt[0],fnt,240,210,_("Use + and - to adjust."));
		ph_draw(gfx,cfg);
		if (key[KEY_PLUS_PAD] && svol<=90) svol+=10;
		if (key[KEY_MINUS_PAD] && svol>=20) svol-=10;
		#if HAVE_ALDUMB_H
			if (cfg->music && cfg->enable_sound) al_poll_duh(snd->music->mpl);
		#endif

	}

	set_volume((svol*255)/100,0);	
	#if HAVE_ALDUMB_H
		aux=svol;
		aux/=100;
		al_duh_set_volume(snd->music->mpl,aux);
	#endif
	destroy_bitmap(bmp);
}

