/*
	texter.c
	Author: Traperic <traperic@alamin.org>, (C) 2005.
	Copyright: See COPYING file that comes with this distribution.
*/

#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <ctype.h>

#include "texter.h"
#include "log.h"
#include "trans.h"


PHFONT * ph_init_texter(void)
{
PHFONT *aux;

	ph_log(_("- Initiating fonts: "));
	aux=(struct PHFONT *)malloc(sizeof(struct PHFONT));
	if (aux==NULL) 
	{
		ph_log_memory_error("texter.c","ph_init_texter","aux",sizeof(struct PHFONT));	
	}
	else
	{
		ph_log(_("Ok.\n"));	
	}
	
	#if HAVE_ALFONT_H
		aux->fnt=NULL;
		aux->ufnt=NULL;
		aux->d1=NULL;
		aux->d2=NULL;
	#else
		aux->fnt=aux->ufnt=font;
	#endif
	
	memset(aux,0,sizeof(struct PHFONT));
	
	return aux;
}

/*! FIXME Con alfont 1.9.2 no da problemas pero con alfont 2 da problemas
al liberar la memoria usada por las fuentes.
*/
void ph_destroy_texter(PHFONT *pfont)
{
	#if HAVE_ALFONT_H
		alfont_destroy_font(pfont->fnt);	
		alfont_destroy_font(pfont->ufnt);	
	#endif

	free(pfont);	
}


void ph_set_text_properties(PHFONT *pf, int color, int back, char fonttype, char align, unsigned int size, char scalable,char aa)
{
	pf->color=color;
	pf->back=back;
	pf->fonttype=fonttype;
	pf->align=align;
#if HAVE_ALFONT_H	
	pf->size=size;
	pf->scalable=scalable;
	pf->aa=aa;
	if (fonttype==TXT_TITLE) alfont_set_font_size(pf->fnt,size);
#else
	pf->size=0;
	pf->scalable=0;
	pf->aa=0;
#endif	
}


void ph_text(BITMAP *bmp, PHFONT *pf, signed short int x, signed short int y, char *buffer)
{
#if HAVE_ALFONT_H
	ALFONT_FONT *f;
#else
	FONT *f;
#endif
	
	if (pf->fonttype==TXT_TITLE) f=pf->fnt; else f=pf->ufnt;
	
	#if HAVE_ALFONT_H
		if (!pf->align && !pf->aa) alfont_textout_ex(bmp,f,buffer,x,y,pf->color,pf->back);
		switch(pf->align)
		{
			case TXT_LEFT:
				if (!pf->aa) alfont_textout_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				else alfont_textout_aa_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
			case TXT_CENTER:
				if (!pf->aa) alfont_textout_centre_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				else alfont_textout_centre_aa_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
			case TXT_RIGHT:
				if (!pf->aa) alfont_textout_right_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				else alfont_textout_right_aa_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
		}
	#else
		if (!pf->align && !pf->aa) textout_ex(bmp,f,buffer,x,y,pf->color,pf->back);
		switch(pf->align)
		{
			case TXT_LEFT:
				textout_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
			case TXT_CENTER:
				textout_centre_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
			case TXT_RIGHT:
				textout_right_ex(bmp,f,buffer,x,y,pf->color,pf->back);
				break;
		}
	#endif
}


int ph_text_len(PHFONT *pfont, char *buffer)
{
int len=0;

	#if HAVE_ALFONT_H
		if (pfont->fonttype==TXT_TITLE)
			len=alfont_text_length(pfont->fnt,buffer);
		else
			len=alfont_text_length(pfont->ufnt,buffer);
		return len;
	#else
		if (pfont->fonttype==TXT_TITLE)
			len=text_length(pfont->fnt, buffer);
		else
			len=text_length(pfont->ufnt, buffer);
		return len;
	#endif
}


void ph_printf(BITMAP *bmp, PHFONT *pf, signed short int x, signed short int y, char *buf, ...)
{
va_list args;
char *buffer;
#if HAVE_ALFONT_H
	ALFONT_FONT *f;
#else
	FONT *f;
#endif

	va_start(args,buf);
	buffer=(char *)malloc(512);
	memset(buffer,0,sizeof(char)*512);
	vsnprintf(buffer,512,buf,args);

	if (pf->fonttype==TXT_TITLE) f=pf->fnt; else f=pf->ufnt;
	
	#if HAVE_ALFONT_H
		if (!pf->align && !pf->aa) alfont_textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
		switch(pf->align)
		{
			case TXT_LEFT:
				if (!pf->aa) alfont_textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				else alfont_textprintf_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_CENTER:
				if (!pf->aa) alfont_textprintf_centre_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				else alfont_textprintf_centre_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_RIGHT:
				if (!pf->aa) alfont_textprintf_right_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				else alfont_textprintf_right_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
		}
	#else
		if (!pf->align && !pf->aa) textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
		switch(pf->align)
		{
			case TXT_LEFT:
				textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_CENTER:
				textprintf_centre_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_RIGHT:
				textprintf_right_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
		}
	#endif
	
	va_end(args);
	free(buffer);
}


void ph_shprintf(BITMAP *bmp, PHFONT *pf, signed short int x, signed short int y, char *buf, ...)
{
va_list args;
char *buffer;
#if HAVE_ALFONT_H
	ALFONT_FONT *f;
#else
	FONT *f;
#endif

	va_start(args,buf);
	buffer=(char *)malloc(512);
	memset(buffer,0,sizeof(char)*512);
	vsnprintf(buffer,512,buf,args);

	if (pf->fonttype==TXT_TITLE) f=pf->fnt; else f=pf->ufnt;
	
	#if HAVE_ALFONT_H
		if (!pf->align && !pf->aa) 
		{
			alfont_textprintf_ex(bmp,f,x+2,y+2,makecol(30,30,30),pf->back,buffer);
			alfont_textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
		}
		switch(pf->align)
		{
			case TXT_LEFT:
				if (!pf->aa) alfont_textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				else alfont_textprintf_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_CENTER:
				if (!pf->aa) 
				{
					alfont_textprintf_centre_ex(bmp,f,x+2,y+2,makecol(30,30,30),pf->back,buffer);
					alfont_textprintf_centre_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				}
				else 
				{
					alfont_textprintf_centre_aa_ex(bmp,f,x+2,y+2,makecol(30,30,30),pf->back,buffer);
					alfont_textprintf_centre_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				}
				break;
			case TXT_RIGHT:
				if (!pf->aa) alfont_textprintf_right_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				else alfont_textprintf_right_aa_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
		}
	#else
		if (!pf->align && !pf->aa) textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
		switch(pf->align)
		{
			case TXT_LEFT:
				textprintf_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_CENTER:
				textprintf_centre_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
			case TXT_RIGHT:
				textprintf_right_ex(bmp,f,x,y,pf->color,pf->back,buffer);
				break;
		}
	#endif
	
	va_end(args);
	free(buffer);
}

char * ph_word_wrap(char *string, size_t line_len)
{
size_t len, current_len = 0;            
size_t start_line = 0;
	  
	while (0 != (len = ph_get_word(&string[current_len + start_line])))
	{
		if (current_len + len < line_len)
			current_len += len;
		else
		{
			string[start_line+current_len] = '\n';
			start_line += current_len + 1;
			current_len = 0;
		}
	}
	return string;
}


size_t ph_get_word(char *string)
{
register int i = 0, word_len = 0;

	if (!string[0]) return 0;

	while (isspace(string[i]))
	{
		if ('\t' == string[i])
		{
			string[i] = ' ';
		}
		else if ('\n' == string[i])
			string[i]=' ';
		word_len++;
		i++;
	}
	while (string[i] && !isspace(string[i++])) word_len++;
	return word_len;
}


/**
 * Esta funcion se encarga de poner en pantalla los textos largos que hay
 * que acortarlos, primero los acorta a la cantidad de caracteres que se
 * especifiquen en la funcion ph_word_wrap, esta funcion pone retornos de
 * carro cada x caracteres respetando espacios y palabras. Despues se
 * escanea el texto en busca de estos retornos de carro y las cadenas
 * comprendidas entre ellos se meten en una lista enlazada. Finalmente se
 * recorre la lista mostrando estas cadenas.
 * @param bmp Bitmap donde mostrar el texto.
 * @param pf Fuente que hay que usar.
 * @param x Coordenada x donde comenzar a poner el texto.
 * @param y Coordenada y donde comenzar a poner el texto.
 * @param buf Cadena sin tratar (solamente tratada con gettext).
 */
void ph_wrap_printf(BITMAP *bmp, PHFONT *pf, signed short int x, signed short int y, char *buf, ...)
{
char *buffer;

	buffer=(char *)malloc(sizeof(char)*(strlen(buf)+100));
	if (buffer==NULL) 
	{
		ph_log_memory_error("texter.c","ph_wrap_printf","buffer",sizeof(char)*(strlen(buf)+100));
	}
	memset(buffer,0,sizeof(char)*(strlen(buf)+100));
	buffer=ph_word_wrap(buf,20);

	free(buffer);
}


